#!/bin/bash
#
#***************************************************************************
# Author: liwanggui
# Email: liwanggui@163.com
# Date: 2021-11-25
# FileName: iotapi_exporter.sh
# Description: iot 第三方接口状态监控数据采集
# Copyright (C): 2021 All rights reserved
#***************************************************************************
#

PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/local/bin:/usr/local/sbin

prog="iotapi"
base_dir=$(dirname $(readlink -f $0))
script="${base_dir}/include/IotSpApiCheck.php"
textfile="/usr/local/node_exporter/metrics/${prog}.prom"

declare -A metrics=()

yiqiying_apis=(b2b2cGetDevicePackageList b2b2cGetTrafficBalance b2b2cIccidValidate)
yiqiying_code=6
yiqiying_zh="翼企盈"

mingyi_apis=(myGetDevicePackageList myGetTrafficBalance myIccidValidate)
mingyi_code=4
mingyi_zh="鸣逸"

# 接口平台列表
platforms=(yiqiying mingyi)

if [[ ! -f $script ]];then
    echo "$script not fount."
    exit 1
fi

function exporter() {
    for platform in ${platforms[*]}; do
        apis=$(eval "echo \${${platform}_apis[*]}")
        zh=$(eval "echo \${${platform}_zh}")
        code=$(eval "echo \${${platform}_code}")

        for api in $apis; do
            status=$(/usr/bin/php $script --method=${api}--sp=${code})
            metrics["iotapi_third_status{platform=\"$zh\", api=\"$api\"}"]=$status
        done
    done
}

function run() {
    [ -f "${textfile}" ] && /bin/rm -f ${textfile}

    if [[ ! -f $script ]];then
        echo "$script not fount."
        exit 1
    fi

    # 获取监控数据
    exporter

    # 写入文件
    OLD=$IFS
    IFS=$'\n'
    for key in ${!metrics[@]}; do
        echo "$key ${metrics[$key]}" >> ${textfile}
    done
    IFS=$OLD
}

function help_usage() {
    cat <<EOF
usage: $0 CMD [options]

commands:
    run              采集 ${prog} 监控数据
    help             打印帮助信息

examples:
    # 运行程序采集监控数据
    $0 run
EOF
}

case "$1" in
    run)
        run
        ;;
    help)
        help_usage
        ;;
    *)
        echo $"Usage: $0 {run|help}"
        exit 2
        ;;
esac