<?php
/**
 * 可独立运行的脚本,不依赖以其它组件.用于给zabbix监控重要接口的可用性
 * 检查代运营商平台检测iccid,查询流量,查询卡已购套餐等接口状态
 * 运行方式 php 路径/IotSpApiCheck.php --method=iccidValidate--sp=6
 * @author  caihailiang
 * @date    2021/10/25
 */
(new IotSpApiCheck())->main();
class IotSpApiCheck {

	/**
	 * 物联网卡对应代运营商平台
	 */
	private $iccidSpMap = [
		'4' => '89860620160030430014', //鸣逸
		'6' => '8986032143200803608', //B2B2C
	];

    /**
     * 发送企业微信平台参数
     */
    private $url = 'https://qyapi.weixin.qq.com/cgi-bin/';
    private $corpId = 'wwe5121c603c79321b';
    private $secret = 'BKbUqVqpvy7gDvgItZKCyYDw3JuP1godp1UTWwosaNw';

    /**
     * 鸣逸平台接口参数
     */
    private $myBaseUrl = 'http://www.unicom-iot.cn:8088/';
    private $myUsername = 'caohuihong';
    private $myPassword = 'acKWdf83D!';
    private $myMsg = [
        'myIccidValidate' => '外部故障警告:鸣逸平台,iccid合法性校检请求失败,将导致无法查询流量,无法充值等,需尽快通知平台方处理',
        'myGetTrafficBalance' => '外部故障警告:鸣逸平台,查询物联网卡用量请求失败,将导致无法查询流量,需尽快通知平台方处理',
        'myGetDevicePackageList' => '外部故障警告:鸣逸平台,获取卡已购买的套餐列表,将导致无法查询卡已购套餐,需尽快通知平台方处理',
    ];

    /**
     * B2B2C平台接口参数
     */
    private $b2b2cBaseUrl = 'https://b2b2c-api.ctwing.cn:9603/';
    private $b2b2cShopIp = 5104;
    private $b2b2cUserName = 'JuanCloud';
    private $b2b2cAppKey = 'J2S5L03r9J1';
    private $b2b2cSecret = 'rlo6ddojNA';
    private $b2b2cMsg = [
        'b2b2cIccidValidate' => '外部故障警告:B2B2C平台,iccid合法性校检请求失败,将导致无法查询流量,无法充值等,需尽快通知平台方处理',
        'b2b2cGetTrafficBalance' => '外部故障警告:B2B2C平台,查询物联网卡用量请求失败,将导致无法查询流量,需尽快通知平台方处理',
        'b2b2cGetDevicePackageList' => '外部故障警告:B2B2C平台,获取卡已购买的套餐列表,将导致无法查询卡已购套餐,需尽快通知平台方处理',
    ];

    /**
     * 监控运营商平台接口
     */
	public function main() {
		global $argv;
		if(!isset($argv[1])){
            echo 0;
            exit;
		}
		$param = $this->parseCommonLineParam($argv[1]);
		//检验参数
		if(empty($param['method']) || empty($param['sp']) || empty($this->iccidSpMap[$param['sp']])){
            echo 0;
            exit;
		}
        $msg = 4 == $param['sp'] ? $this->myMsg : $this->b2b2cMsg;
        $method = $param['method'];
        //请求的方法没有设置告警消息,报错
        if(empty($msg[$method])){
            echo 0;
            exit;
        }
        //请求的方法不存在
        if(!method_exists($this,$param['method'])){
            echo 0;
            exit;
        }
        $res = $this->$method($this->iccidSpMap[$param['sp']]);
        if(!$res['ack']){
            //接口请求失败,发送企业微信通知
            $this->sendMessage($msg[$param['method']]);
            echo 0;
            exit;
        }
       echo 1;
	}

    /**
     * 鸣逸iccid合法性校检
     */
    private function myIccidValidate($iccid) {
        $res = $this->myPost('api/verifyDevice', ['iccid' => $iccid]);
        if (!$res['ack']) {
            return ['ack' => false, 'msg' => $res['msg']];
        }
        return ['ack' => true, 'iccid' => $res['data']['iccid']];
    }

    /**
     * 鸣逸查询物联网卡用量
     */
    private function myGetTrafficBalance($iccid) {
        $res = $this->myPost('api/getDeviceUsage', ['iccid' => $iccid]);
        if (!$res['ack']) {
            return ['ack' => false, 'msg' => $res['msg']];
        }
        return ['ack' => true];
    }

    /**
     * 鸣逸获取卡已购买的套餐列表
     */
    private function myGetDevicePackageList($iccid) {
        $res = $this->myPost('api/getRenewlistList', ['iccid' => $iccid]);
        if (!$res['ack']) {
            return ['ack' => false, 'msg' => $res['msg']];
        }
        return ['ack' => true];
    }

    /**
     * 鸣逸post请求
     */
    private function myPost($action, $data = []) {
        try {
            $data = array_merge($data,['username' => $this->myUsername,'password' => $this->myPassword]);
            $response = $this->send($this->myBaseUrl.$action, http_build_query($data));
        } catch (Exception $e) {
            return [
                'ack' => false,
                'msg' => 'network request failed',
                'errorInfo' => json_decode($e->getMessage(), true),
            ];
        }
        $ret = json_decode($response, true);
        if (empty($ret)) {
            return ['ack' => false, 'msg' => 'mingyi return form error'];
        }
        if (!isset($ret['success']) || !$ret['success']) {
            $errorMsg = isset($ret['msg']) ? $ret['msg'] : 'unknow mingyi error msg';
            $errorCode = isset($ret['errorCode']) ? $ret['errorCode'] : 'unknow mingyi error code';
            return ['ack' => false, 'msg' => $errorMsg.'|'.$errorCode];
        }
        return ['ack' => true, 'data' => $ret];
    }

    /**
     * b2b2ciccid合法性校检
     */
    private function b2b2cIccidValidate($iccid) {
        $data = [
            "paramType" => "CARD",
            "paramValue" => $iccid,
            "userName" => $this->b2b2cUserName,
        ];
        $res = $this->b2b2cPost('openapi/api/v1/mktResInst/OpenApi/getCardDetail', json_encode($data));
        if (!$res['ack']) {
            return ['ack' => false, 'msg' => $res['msg']];
        }
        return ['ack' => true];
    }

    /**
     * b2b2c查询物联网卡用量
     */
    private function b2b2cGetTrafficBalance($iccid) {
        $res = $this->b2b2cPost('openapi/api/vi/accu/qryRatable', '', ['cardOrAccNbr' => $iccid,'shopId' => $this->b2b2cShopIp], 2);
        if (!$res['ack']) {
            return ['ack' => false, 'msg' => $res['msg']];
        }
        return ['ack' => true];
    }

    /**
     * b2b2c获取卡已购买的套餐列表
     */
    private function b2b2cGetDevicePackageList($iccid) {
        $res = $this->b2b2cPost('openapi/qryOfferInst', '', [
            'cardOrAccNbr' => $iccid,
            'shopId' => $this->b2b2cShopIp,
        ], 2);
        if (!$res['ack']) {
            return ['ack' => false, 'msg' => $res['msg']];
        }
        return ['ack' => true];
    }

    /**
     * b2b2c post请求
     * @param $action
     * @param  string  $body  请求内容
     * @param  array  $queryParams  url请求参数
     * @param  int  $sendType  请求方式 1 post 2 get
     * @return array
     */
    private function b2b2cPost($action, $body = '', $queryParams = [], $sendType = 1) {
        try {
            $signRes = $this->createSign($body, $queryParams);
            if (!$signRes['ack']) {
                return ['ack' => false, 'msg' => $signRes['msg']];
            }
            $url = $this->b2b2cBaseUrl.$action;
            $url .= empty($queryParams) ? '' : '?'.http_build_query($queryParams);
            $headers = [
                'Content-Type: application/json; charset=utf-8',
                'application:'.$signRes['data']['application'],
                'timestamp:'.$signRes['data']['timestamp'],
                'signature:'.$signRes['data']['signature'],
            ];
            $response = $this->send($url, $body, 4, $sendType, $headers);
        } catch (Exception $e) {
            return [
                'ack' => false,
                'msg' => 'network request failed',
                'errorInfo' => json_decode($e->getMessage(), true),
            ];
        }
        $ret = json_decode($response, true);
        if (empty($ret)) {
            return ['ack' => false, 'msg' => 'yiqiying return form error'];
        }
        if (!isset($ret['code']) || 0 != $ret['code']) {
            $errorMsg = isset($ret['message']) ? $ret['message'] : 'unknow yiqiying error msg';
            $errorCode = isset($ret['code']) ? $ret['code'] : 'unknow yiqiying error code';
            return ['ack' => false, 'msg' => $errorMsg.'|'.$errorCode];
        }
        return ['ack' => true, 'data' => $ret];
    }

    /**
     * 生成请求签名
     * @param  string  $body  请求内容
     * @param  array  $queryParams  url请求参数
     * @return array
     */
    private function createSign($body = '', $queryParams = []) {
        // 公共参数
        $commonData = [
            'application' => $this->b2b2cAppKey,
            'timestamp' => $this->createMicroTimeStamp(),
        ];
        $str = '';
        if ($queryParams) {
            ksort($queryParams);
            $commonData = array_merge($commonData, $queryParams);
        }
        foreach ($commonData as $key => $item) {
            $str .= $key.":".$item."\n";
        }
        if ($body) {
            $str .= $body."\n";
        }
        $encode = hash_hmac("sha1", $str, $this->b2b2cSecret, true);
        $signature = base64_encode($encode);
        $commonData['signature'] = $signature;
        return ['ack' => true, 'data' => $commonData];
    }

    /**
     * 生成毫秒级时间戳(13位字符串)
     */
    private  function createMicroTimeStamp(){
        $arr = explode(' ', microtime());
        return sprintf('%.0f',(floatval($arr[0])+floatval($arr[1]))*1000);
    }

	/**
	 * 解析命令行中自定义的参数格式
	 */
	private function parseCommonLineParam($param){
		$arr = explode('--',$param);
		$rtn = array();
		foreach($arr as $v){
			if(!empty($v)){
				$temp = explode('=',$v);
				$rtn[$temp[0]] = isset($temp[1]) ? $temp[1] : null;
			}
		}
		return $rtn;
	}

    /**
     * 获取请求token
     */
    private function getToken(){
        $data = [
            'corpid'=>$this->corpId,
            'corpsecret'=>$this->secret
        ];
        $url = $this->url.'gettoken';
        try {
            $response = $this->send($url,json_encode($data),2);
            $res = json_decode($response,true);
            if (0 != $res['errcode']) {
                return array('ack' => false,'msg' => $res['msg']);
            }
            return ['ack'=>true,'data'=>$res['access_token']];
        } catch (Exception $e) {
            return ['ack' => false, 'msg' => 'Network request failed'];
        }
    }

    /**
     * 发送消息
     * @param string $touser
     * @param string $message
     * @return array
     */
    private function sendMessage($message,$touser = '@all'){
        $getTokenRes = $this->getToken();
        if (!$getTokenRes['ack']){
            return ['ack'=>false,'mag'=>$getTokenRes['msg']];
        }
        $url = $this->url.'message/send?access_token='.$getTokenRes['data'];
        $obj = new stdClass();
        $obj->content = $message;
        $data = [
            'touser'=>$touser,
            'toparty'=>2,
            'msgtype'=>'text',
            'agentid'=>1000012,
            'text'=>$obj,
            'safe'=>0
        ];
        try {
            $response = $this->send($url,json_encode($data,256),2);
            $res = json_decode($response,true);
            if (0 != $res['errcode']) {
                return array('ack' => false,'msg' => $res['msg']);
            }
            return ['ack'=>true,'data'=>'ok'];
        } catch (Exception $e) {
            return ['ack' => false, 'msg' => 'Network request failed'];
        }
    }

    /**
     * 发送请求
     * @param string $url
     * @param string|array $data 发送的数据;如果是get发送,数据由调用者拼接在url后
     * @param int $contenType 1:键值;2:json,3:不设置,4:自定义contentType和自定义头
     * @param int $sendType 1:post,2:get
     * @param array $headers 请求头信息
     * @param int $timeOut 请求超时时间
     * @return bool|string
     * @throws Exception 调用者需要自己捕获异常
     */
    private function send($url,$data,$contenType = 1,$sendType = 1,$headers = array(),$timeOut = 10){
        $ch = curl_init ();
        curl_setopt ( $ch, CURLOPT_URL, $url );
        if(1 == $sendType){
            curl_setopt ( $ch, CURLOPT_POST, true );
            curl_setopt ( $ch, CURLOPT_POSTFIELDS, $data );
        }else{
            curl_setopt ( $ch, CURLOPT_POST, false );
        }
        curl_setopt ( $ch, CURLOPT_RETURNTRANSFER, 1 );
        curl_setopt ( $ch, CURLOPT_TIMEOUT, $timeOut );
        curl_setopt ( $ch, CURLOPT_SSL_VERIFYPEER, false );
        curl_setopt ( $ch, CURLOPT_SSL_VERIFYHOST, false );
        if(1 == $contenType){
            curl_setopt ( $ch, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded; charset=utf-8'));
        }elseif(2 == $contenType){
            curl_setopt ( $ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json; charset=utf-8'));
        }elseif(4 == $contenType){
            curl_setopt ( $ch, CURLOPT_HTTPHEADER, $headers);
        }
        $returnData = curl_exec ( $ch );
        $httpData = curl_getinfo ( $ch );
        // HTTP请求不成功
        if (200 != $httpData ['http_code']) {
            $error = json_encode ( $httpData );
            throw new Exception($error);
        }
        return $returnData;
    }

}
